@@ -1,10 +1,10 @@
 package Algorithm::Simplex::PDL;
-use Moose;
+use Moo;
+use MooX::Types::MooseLike::Base qw( ArrayRef Str );
 extends 'Algorithm::Simplex';
 with 'Algorithm::Simplex::Role::Solve';
-use Algorithm::Simplex::Types;
 use PDL::Lite;
-use namespace::autoclean;
+use namespace::clean;
 
 =head1 Name
 
@@ -15,13 +15,13 @@ Algorithm::Simplex::PDL - PDL model of the Simplex Algorithm
 # TODO: Probably need EPSILON for zero approximation check like in Float model.
 
 has '+tableau' => (
-    isa        => 'Piddle',
-    coerce     => 1,
+    isa    => sub { $_[0]->isa('PDL') },
+    coerce => sub { PDL->pdl($_[0]) },
 );
 
 has '+display_tableau' => (
-    isa        => 'PiddleDisplay',
-    coerce     => 1,
+    isa => ArrayRef [ ArrayRef [Str] ],
+    coerce => sub { &display_piddle($_[0]) },
 );
 
 =head1 Methods
@@ -36,7 +36,7 @@ The same holds for number of columns.
 
 sub _build_number_of_rows {
     my $self = shift;
-    my ( $number_of_columns, $number_of_rows ) = ( $self->tableau->dims);
+    my ($number_of_columns, $number_of_rows) = ($self->tableau->dims);
     return $number_of_rows - 1;
 }
 
@@ -48,7 +48,7 @@ set the number of columns given the tableau matrix
 
 sub _build_number_of_columns {
     my $self = shift;
-    my ( $number_of_columns, $number_of_rows ) = ( $self->tableau->dims );
+    my ($number_of_columns, $number_of_rows) = ($self->tableau->dims);
     return $number_of_columns - 1;
 }
 
@@ -71,16 +71,15 @@ sub pivot {
 
     my $scale_copy =
       $pdl_A->slice("($pivot_column_number),($pivot_row_number)")->copy;
-    my $scale = $pdl_A->slice("($pivot_column_number),($pivot_row_number)");
+    my $scale     = $pdl_A->slice("($pivot_column_number),($pivot_row_number)");
     my $pivot_row = $pdl_A->slice(":,($pivot_row_number)");
     $pivot_row /= $scale_copy;
     $scale     /= $scale_copy;
 
     # peform pivot algebra in non-pivot rows
-    for my $i ( 0 .. $self->number_of_rows ) {
-        if ( $i != $pivot_row_number ) {
-            my $a_ic_copy =
-              $pdl_A->slice("($pivot_column_number),($i)")->copy;
+    for my $i (0 .. $self->number_of_rows) {
+        if ($i != $pivot_row_number) {
+            my $a_ic_copy  = $pdl_A->slice("($pivot_column_number),($i)")->copy;
             my $a_ic       = $pdl_A->slice("($pivot_column_number),($i)");
             my $change_row = $pdl_A->slice(":,($i)");
             my $diff_term  = $a_ic x $pivot_row;
@@ -94,10 +93,10 @@ sub pivot {
     return $pdl_A;
 }
 
-# Count pivots made 
+# Count pivots made
 after 'pivot' => sub {
     my $self = shift;
-    $self->number_of_pivots_made( $self->number_of_pivots_made + 1 );
+    $self->number_of_pivots_made($self->number_of_pivots_made + 1);
     return;
 };
 
@@ -116,9 +115,8 @@ sub is_optimal {
     my $number_of_rows = $self->number_of_rows;
     my $basement_row   = $T_pdl->slice("0:$n_cols_A,($number_of_rows)");
     my @basement_row   = $basement_row->list;
-    my @positive_profit_column_numbers;
     foreach my $profit_coefficient (@basement_row) {
-        if ( $profit_coefficient > 0 ) {
+        if ($profit_coefficient > 0) {
             return 0;
         }
     }
@@ -143,12 +141,12 @@ sub determine_simplex_pivot_columns {
     my @simplex_pivot_column_numbers;
     my $n_cols_A       = $self->number_of_columns - 1;
     my $number_of_rows = $self->number_of_rows;
-    my $basement_row = $self->tableau->slice("0:$n_cols_A,($number_of_rows)");
-    my @basement_row = $basement_row->list;
-    my $column_number = 0;
+    my $basement_row   = $self->tableau->slice("0:$n_cols_A,($number_of_rows)");
+    my @basement_row   = $basement_row->list;
+    my $column_number  = 0;
     foreach my $profit_coefficient (@basement_row) {
 
-        if ( $profit_coefficient > 0 ) {
+        if ($profit_coefficient > 0) {
             push @simplex_pivot_column_numbers, $column_number;
         }
         $column_number++;
@@ -180,14 +178,13 @@ sub determine_positive_ratios {
     my @positive_ratio_row_numbers;
     my @positive_ratios;
 
-    foreach my $i ( 0 .. $n_rows_A ) {
-        if ( $pivot_column[$i] > 0 ) {
-            push @positive_ratios,
-              ( $constant_column[$i] / $pivot_column[$i] );
+    foreach my $i (0 .. $n_rows_A) {
+        if ($pivot_column[$i] > 0) {
+            push @positive_ratios, ($constant_column[$i] / $pivot_column[$i]);
             push @positive_ratio_row_numbers, $i;
         }
     }
-    return ( \@positive_ratios, \@positive_ratio_row_numbers );
+    return (\@positive_ratios, \@positive_ratio_row_numbers);
 }
 
 =head2 display_pdl
@@ -223,23 +220,44 @@ sub current_solution {
       $self->tableau->slice("($number_of_columns),0:$n_rows_A");
     my @constant_column = $constant_column->list;
     my %primal_solution;
-    for my $i ( 0 .. $#y ) {
+    for my $i (0 .. $#y) {
         $primal_solution{ $y[$i]->{generic} } = $constant_column[$i];
     }
 
     # Dependent Dual Variables
     my $n_cols_A       = $self->number_of_columns - 1;
     my $number_of_rows = $self->number_of_rows;
-    my $basement_row = $self->tableau->slice("0:$n_cols_A,($number_of_rows)");
-    my @basement_row = $basement_row->list;
+    my $basement_row   = $self->tableau->slice("0:$n_cols_A,($number_of_rows)");
+    my @basement_row   = $basement_row->list;
     my %dual_solution;
-    for my $j ( 0 .. $#u ) {
+    for my $j (0 .. $#u) {
         $dual_solution{ $u[$j]->{generic} } = $basement_row[$j] * (-1);
     }
 
-    return ( \%primal_solution, \%dual_solution );
+    return (\%primal_solution, \%dual_solution);
 }
 
-__PACKAGE__->meta->make_immutable;
+=head2 display_piddle 
+
+Coercion:  convert a PDL into an ArrayRef[ArrayRef[Num]]
+
+=cut
+
+sub display_piddle {
+    my $piddle_tableau = shift;
+
+    my @display_tableau;
+    my ($number_of_columns, $number_of_rows) = ($piddle_tableau->dims);
+    my $number_of_zero_based_rows    = $number_of_rows - 1;
+    my $number_of_zero_based_columns = $number_of_columns - 1;
+    for my $i (0 .. $number_of_zero_based_rows) {
+        my $row =
+          $piddle_tableau->slice("0:$number_of_zero_based_columns,($i)");
+        my @row = $row->list;
+        push @display_tableau, \@row;
+    }
+
+    return \@display_tableau;
+}
 
 1;
